@@ -36,12 +36,12 @@
 #include "channel.h"
 #include "chansession.h"
 #include "ssh.h"
+#include "x11fwd.h"
+#include "agentfwd.h"
 
-static struct Channel* newchannel(unsigned int remotechan, char type, 
-		unsigned int transwindow, unsigned int transmaxpacket);
 static void send_msg_channel_open_failure(unsigned int remotechan, int reason,
 		const unsigned char *text, const unsigned char *lang);
-static void send_msg_channel_open_success(struct Channel* channel,
+static void send_msg_channel_open_confirmation(struct Channel* channel,
 		unsigned int recvwindow, 
 		unsigned int recvmaxpacket);
 static void writechannel(struct Channel *channel);
@@ -79,15 +79,16 @@ void chancleanup() {
 	m_free(ses.channels);
 	TRACE(("leave chancleanup"));
 }
-			
 
 /* create a new channel entry, send a reply confirm or failure */
-static struct Channel* newchannel(unsigned int remotechan, char type, 
+/* If remotechan, transwindow and transmaxpacket are not know (for a new
+ * outgoing connection, with them to be filled on confirmation), they should
+ * all be set to 0 */
+struct Channel* newchannel(unsigned int remotechan, unsigned char type, 
 		unsigned int transwindow, unsigned int transmaxpacket) {
 
-	struct Channel ** chanlist;
 	struct Channel * newchan;
-	int i;
+	int i, j;
 
 	TRACE(("enter newchannel"));
 	
@@ -101,18 +102,21 @@ static struct Channel* newchannel(unsigned int remotechan, char type,
 	/* otherwise extend the list */
 	if (i == ses.chansize) {
 		if (ses.chansize > MAX_CHANNELS) {
-			send_msg_channel_open_failure(remotechan,
-					SSH_OPEN_RESOURCE_SHORTAGE, "", "");
 			TRACE(("leave newchannel: max chans reached"));
 			return NULL;
 		}
 
 		/* extend the channels */
-		chanlist = (struct Channel**)m_realloc(ses.channels,
+		ses.channels = (struct Channel**)m_realloc(ses.channels,
 				(ses.chansize+CHAN_EXTEND_SIZE)*sizeof(struct Channel*));
 
-		ses.channels = chanlist;
 		ses.chansize += CHAN_EXTEND_SIZE;
+
+		/* set the new channels to null */
+		for (j = i; j < ses.chansize; j++) {
+			ses.channels[j] = NULL;
+		}
+
 	}
 	
 	newchan = (struct Channel*)m_malloc(sizeof(struct Channel));
@@ -120,9 +124,11 @@ static struct Channel* newchannel(unsigned int remotechan, char type,
 	newchan->index = i;
 	newchan->sentclosed = 0;
 	newchan->recveof = newchan->transeof = newchan->erreof = 0;
+
 	newchan->remotechan = remotechan;
 	newchan->transwindow = transwindow;
 	newchan->transmaxpacket = transmaxpacket;
+	
 	newchan->typedata = NULL;
 	newchan->infd = -1;
 	newchan->outfd = -1;
@@ -134,8 +140,6 @@ static struct Channel* newchannel(unsigned int remotechan, char type,
 
 	ses.channels[i] = newchan;
 
-	send_msg_channel_open_success(newchan, RECV_MAXWINDOW,
-			RECV_MAXPACKET);
 	TRACE(("leave newchannel"));
 
 	return newchan;
@@ -175,7 +179,23 @@ void channelio(fd_set *readfd, fd_set *writefd) {
 				FD_ISSET(channel->infd, writefd)) {
 			writechannel(channel);
 		}
-	}
+		
+		/* handle any listening sockets */
+		if (channel->type == CHANNEL_ID_SESSION) {
+			struct ChanSess * chansess = (struct ChanSess*)channel->typedata;
+#ifndef DISABLE_X11FWD
+			if (chansess->x11fd != -1 && FD_ISSET(chansess->x11fd, readfd)) {
+				x11accept(chansess);
+			}
+#endif
+#ifndef DISABLE_AGENTFWD
+			if (chansess->agentfd != -1 && FD_ISSET(chansess->agentfd,readfd)) {
+				agentaccept(chansess);
+			}
+#endif
+		}
+	} /* foreach channel */
+
 }
 
 static void send_exitsignalstatus(struct Channel *channel) {
@@ -225,10 +245,9 @@ static void send_msg_channel_eof(struct Channel *channel) {
 	/* we already know that trans/eof channels are closed */
 	send_msg_channel_close(channel);
 
-
 	TRACE(("leave send_msg_channel_eof"));
-
 }
+
 /* only called when we know we can write to a channel, writes as much as
  * possible */
 static void writechannel(struct Channel* channel) {
@@ -310,7 +329,24 @@ void setchannelfds(fd_set *readfd, fd_set *writefd) {
 			/* there's space to write more to the program */
 			FD_SET(channel->infd, writefd);
 		}
-	}
+
+		/* handle any listening sockets - should get optimised away if
+		 * we don't have x11 or agent fwd */
+		if (channel->type == CHANNEL_ID_SESSION) {
+			struct ChanSess * chansess = (struct ChanSess*)channel->typedata;
+#ifndef DISABLE_X11FWD
+			if (chansess->x11fd != -1) {
+				FD_SET(chansess->x11fd, readfd);
+			}
+#endif
+#ifndef DISABLE_AGENTFWD
+			if (chansess->agentfd != -1) {
+				FD_SET(chansess->agentfd, readfd);
+			}
+#endif
+		}
+	} /* foreach channel */
+
 }
 
 void recv_msg_channel_eof() {
@@ -330,8 +366,12 @@ void recv_msg_channel_eof() {
 	channel->recveof = 1;
 
 	/* we should close the channel */
-	/* XXX when TCP forwarding etc is done, use shutdown() not close() */
-	close(channel->infd);
+	if (channel->type == CHANNEL_ID_X11 || channel->type == CHANNEL_ID_AGENT) {
+		shutdown(channel->infd, 0);
+	} else {
+		close(channel->infd);
+	}
+	channel->infd = -1;
 
 	if (channel->transeof && (channel->erreof || channel->errfd == -1)
 			&& !channel->sentclosed) {
@@ -350,6 +390,7 @@ void recv_msg_channel_close() {
 	TRACE(("enter recv_msg_channel_close"));
 
 	chan = buf_getint(ses.payload);
+	TRACE(("close channel = %d", chan));
 	channel = getchannel(chan);
 
 	if (channel == NULL) {
@@ -368,21 +409,23 @@ void recv_msg_channel_close() {
 
 static void closechannel(struct Channel * channel) {
 
-	unsigned int index;
-
 	TRACE(("enter closechannel"));
 	TRACE(("channel index is %d", channel->index));
 	
 	buf_free(channel->writebuf);
-	TRACE(("frees done "));
+
+	/* close the FDs in case they haven't been done
+	 * yet (ie they were shutdown etc */
+	close(channel->infd);
+	close(channel->outfd);
 
 	if (channel->type == CHANNEL_ID_SESSION) {
 		closechansess(channel);
 	}
 
-	index = channel->index;
+	ses.channels[channel->index] = NULL;
 	m_free(channel);
-	ses.channels[index] = NULL;
+
 	TRACE(("leave closechannel"));
 }
 
@@ -412,8 +455,6 @@ void recv_msg_channel_request() {
 			chansessionrequest(channel);
 			break;
 
-		case CHANNEL_ID_X11:
-
 		default:
 			send_msg_channel_failure(channel);
 	}
@@ -528,7 +569,6 @@ void recv_msg_channel_data() {
 	}
 
 	assert(channel->infd != -1);
-	assert(channel->sentclosed == 0);
 
 	datalen = buf_getint(ses.payload);
 
@@ -568,7 +608,7 @@ void recv_msg_channel_window_adjust() {
 	channel = getchannel(chan);
 
 	if (channel == NULL) {
-		dropbear_exit("Invalid channel"); /* TODO - disconnect */
+		dropbear_exit("Unknown channel"); /* TODO - disconnect */
 	}
 	
 	incr = buf_getint(ses.payload);
@@ -597,44 +637,55 @@ void recv_msg_channel_open() {
 
 	unsigned char* type;
 	unsigned int typelen;
+	unsigned int typeval;
 	unsigned int remotechan, transwindow, transmaxpacket;
 	struct Channel* channel;
+	unsigned int errtype = SSH_OPEN_UNKNOWN_CHANNEL_TYPE;
 
 
 	TRACE(("enter recv_msg_channel_open"));
 
+	/* get the packet contents */
 	type = buf_getstring(ses.payload, &typelen);
 
-	TRACE(("thing here"));
 	remotechan = buf_getint(ses.payload);
-	TRACE(("thing here"));
 	transwindow = buf_getint(ses.payload);
 	transwindow = MIN(transwindow, MAX_TRANS_WINDOW);
-	TRACE(("thing here"));
 	transmaxpacket = buf_getint(ses.payload);
-	TRACE(("thing here"));
 	transmaxpacket = MIN(transmaxpacket, MAX_TRANS_PAYLOAD_LEN);
 
-
+	/* figure what type of packet it is */
 	if (typelen > MAX_NAME_LEN) {
-		/* send channel_open_failure below */
-	} else if (strcmp(type, "session") == 0) {
-			channel = newchannel(remotechan, CHANNEL_ID_SESSION, 
-					transwindow, transmaxpacket);
-			if (channel != NULL) {
-				newchansess(channel);
-			}
-			goto out;
-	} else if (strcmp(type, "x11") == 0) {
-			newchannel(remotechan, CHANNEL_ID_X11,
-					transwindow, transmaxpacket);
-			goto out;
+		goto failure;
+	}
+	if (strcmp(type, "session") == 0) {
+		typeval = CHANNEL_ID_SESSION;
+	} else {
+		goto failure;
 	}
 
-	send_msg_channel_open_failure(remotechan, 
-			SSH_OPEN_UNKNOWN_CHANNEL_TYPE,
-			"Unknown channel", "en");
-out:
+	/* create the channel */
+	channel = newchannel(remotechan, typeval, transwindow, transmaxpacket);
+
+	if (channel == NULL) {
+		errtype = SSH_OPEN_RESOURCE_SHORTAGE;
+		goto failure;
+	}
+	
+	/* type specific initialisation */
+	if (typeval == CHANNEL_ID_SESSION) {
+		newchansess(channel);
+	}
+
+	/* success */
+	send_msg_channel_open_confirmation(channel, channel->recvwindow,
+			channel->recvmaxpacket);
+	goto cleanup;
+
+failure:
+	send_msg_channel_open_failure(remotechan, errtype, "", "");
+
+cleanup:
 	m_free(type);
 
 	TRACE(("leave recv_msg_channel_open"));
@@ -680,11 +731,11 @@ static void send_msg_channel_open_failure(unsigned int remotechan,
 	TRACE(("leave send_msg_channel_open_failure"));
 }
 
-static void send_msg_channel_open_success(struct Channel* channel,
+static void send_msg_channel_open_confirmation(struct Channel* channel,
 		unsigned int recvwindow, 
 		unsigned int recvmaxpacket) {
 
-	TRACE(("enter send_msg_channel_open_success"));
+	TRACE(("enter send_msg_channel_open_confirmation"));
 	CHECKCLEARTOWRITE();
 
 	buf_putbyte(ses.writepayload, SSH_MSG_CHANNEL_OPEN_CONFIRMATION);
@@ -694,5 +745,76 @@ static void send_msg_channel_open_success(struct Channel* channel,
 	buf_putint(ses.writepayload, recvmaxpacket);
 
 	encrypt_packet();
-	TRACE(("leave send_msg_channel_open_success"));
+	TRACE(("leave send_msg_channel_open_confirmation"));
 }
+
+#ifdef USE_LISTENERS
+/* Create a new channel, and start the open request. This is intended
+ * for X11, agent, tcp forwarding, and should be filled with channel-specific
+ * options, with the calling function calling encrypt_packet() after
+ * completion. It is mandatory for the caller to encrypt_packet() if
+ * DROPBEAR_SUCCESS is returned */
+int send_msg_channel_open_init(int fd, const char * typestring) {
+
+	struct Channel* chan;
+
+	chan = newchannel(0, CHANNEL_ID_AGENT, 0, 0);
+	if (!chan) {
+		return DROPBEAR_FAILURE;
+	}
+
+	/* set fd non-blocking */
+	if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0) {
+		return DROPBEAR_FAILURE;
+	}
+
+	chan->infd = chan->outfd = fd;
+	ses.maxfd = MAX(ses.maxfd, fd);
+
+	/* now open the channel connection */
+	CHECKCLEARTOWRITE();
+
+	buf_putbyte(ses.writepayload, SSH_MSG_CHANNEL_OPEN);
+	buf_putstring(ses.writepayload, typestring, strlen(typestring));
+	buf_putint(ses.writepayload, chan->index);
+	buf_putint(ses.writepayload, RECV_MAXWINDOW);
+	buf_putint(ses.writepayload, RECV_MAXPACKET);
+
+	return DROPBEAR_SUCCESS;
+}
+
+/* channel establishment is only required if we have listeners (for x11 etc)*/
+void recv_msg_channel_open_confirmation() {
+
+	unsigned int chan;
+	struct Channel * channel;
+
+	TRACE(("enter recv_msg_channel_open_confirmation"));
+	chan = buf_getint(ses.payload);
+
+	channel = getchannel(chan);
+	if (channel == NULL) {
+		dropbear_exit("Unknown channel");
+	}
+
+	channel->remotechan =  buf_getint(ses.payload);
+	channel->transwindow = buf_getint(ses.payload);
+	channel->transmaxpacket = buf_getint(ses.payload);
+
+	TRACE(("leave recv_msg_channel_open_confirmation"));
+}
+
+void recv_msg_channel_open_failure() {
+
+	unsigned int chan;
+	struct Channel * channel;
+	chan = buf_getbyte(ses.payload);
+
+	channel = getchannel(chan);
+	if (channel == NULL) {
+		dropbear_exit("Unknown channel");
+	}
+
+	closechannel(channel);
+}
+#endif /* USE_LISTENERS */
