@@ -30,6 +30,8 @@
 
 /* Debugging */
 
+/*#define DISABLE_X11FWD*/
+
 /*#define DEBUG_KEXHASH*/
 /*#define DEBUG_RSA*/
 
@@ -40,6 +42,12 @@
 /* Define this to print trace statements */
 /*#define DEBUG_TRACE*/
 
+/* All functions writing to the cleartext payload buffer call
+ * CHECKCLEARTOWRITE() before writing. This is only really useful if you're
+ * attempting to track down a problem */
+#define CHECKCLEARTOWRITE() assert(ses.writepayload->len == 0 && \
+		ses.writepayload->pos == 0)
+
 /* Define this, compile with -pg and set GMON_OUT_PREFIX=gmon to get gmon
  * output when Dropbear forks. This will allow it gprof to be used.
  * It's useful to run dropbear -F, so you don't fork as much */
