@@ -42,6 +42,8 @@
 #include "libtomcrypt/mycrypt.h"
 
 static void read_packet_init();
+static void process_postauth_packet(unsigned int type);
+static void recv_unimplemented();
 
 #define ZLIB_COMPRESS_INCR 20 /* this is 12 bytes + 0.1% of 8000 bytes */
 #define ZLIB_DECOMPRESS_INCR 100
@@ -193,14 +195,10 @@ static void read_packet_init() {
 
 	buf_setpos(ses.readbuf, blocksize);
 
-	/* check packet lengths */
-	if (len > MAX_PACKET_LEN) {
-		dropbear_exit("bad packet size");
-	}
-	if (len < MIN_PACKET_LEN + macsize) {
-		dropbear_exit("bad packet size");
-	}
-	if ((len - macsize) % blocksize != 0) {
+	/* check packet length */
+	if ((len > MAX_PACKET_LEN) ||
+		(len < MIN_PACKET_LEN + macsize) ||
+		((len - macsize) % blocksize != 0)) {
 		dropbear_exit("bad packet size");
 	}
 
@@ -300,10 +298,7 @@ void decrypt_packet() {
 	/* payload length */
 	/* - 4 - 1 is for LEN and PADLEN values */
 	len = ses.decryptreadbuf->len - padlen - 4 - 1;
-	if (len > MAX_PAYLOAD_LEN) {
-		dropbear_exit("bad packet size");
-	}
-	if (len < 1) { /* this is valid? */
+	if ((len > MAX_PAYLOAD_LEN) || (len < 1)) {
 		dropbear_exit("bad packet size");
 	}
 
@@ -408,13 +403,71 @@ void process_packet() {
 
 	/* check that we aren't expecting a particular packet */
 	if (ses.expecting && ses.expecting != type) {
-		/* TODO be more verbose, send disconnect? */
-		dropbear_exit("unexpected packet type %d", type);
+		/* TODO send disconnect? */
+		dropbear_exit("unexpected packet type %d, expected %d", type,
+				ses.expecting);
 	}
 
 	/* handle the packet depending on type */
 	ses.expecting = 0;
 
+	switch (type) {
+
+		case SSH_MSG_SERVICE_REQUEST:
+			recv_msg_service_request();
+			break;
+
+		case SSH_MSG_USERAUTH_REQUEST:
+			recv_msg_userauth_request();
+			break;
+			
+		case SSH_MSG_KEXINIT:
+			recv_msg_kexinit();
+			break;
+
+		case SSH_MSG_KEXDH_INIT:
+			recv_msg_kexdh_init();
+			break;
+
+		case SSH_MSG_NEWKEYS:
+			recv_msg_newkeys();
+			break;
+
+		/* this is ugly, need to make a cleaner way to do it */
+		case SSH_MSG_CHANNEL_DATA:
+		case SSH_MSG_CHANNEL_WINDOW_ADJUST:
+		case SSH_MSG_CHANNEL_REQUEST:
+		case SSH_MSG_CHANNEL_OPEN:
+		case SSH_MSG_CHANNEL_EOF:
+		case SSH_MSG_CHANNEL_CLOSE:
+		case SSH_MSG_CHANNEL_OPEN_CONFIRMATION:
+		case SSH_MSG_CHANNEL_OPEN_FAILURE:
+			/* these should be checked for authdone below */
+			process_postauth_packet(type);
+			break;
+	
+		default:
+			/* TODO this possibly should be handled */
+			TRACE(("unknown packet"));
+			recv_unimplemented();
+			break;
+	}
+
+out:
+	buf_free(ses.payload);
+	ses.payload = NULL;
+
+	TRACE(("leave process_packet"));
+}
+
+/* process a packet, and also check that auth has been done */
+static void process_postauth_packet(unsigned int type) {
+
+	/* messages following here require userauth before use */
+	if (!ses.authstate.authdone) {
+		dropbear_exit("received message %d before userauth", type);
+	}
+
 	switch (type) {
 
 		case SSH_MSG_CHANNEL_DATA:
@@ -441,38 +494,41 @@ void process_packet() {
 			recv_msg_channel_close();
 			break;
 
-		case SSH_MSG_SERVICE_REQUEST:
-			recv_msg_service_request();
-			break;
-
-		case SSH_MSG_USERAUTH_REQUEST:
-			recv_msg_userauth_request();
+#ifdef USE_LISTENERS /* for x11, tcp fwd etc */
+		case SSH_MSG_CHANNEL_OPEN_CONFIRMATION:
+			recv_msg_channel_open_confirmation();
 			break;
 			
-		case SSH_MSG_KEXINIT:
-			recv_msg_kexinit();
+		case SSH_MSG_CHANNEL_OPEN_FAILURE:
+			recv_msg_channel_open_failure();
 			break;
-
-		case SSH_MSG_KEXDH_INIT:
-			recv_msg_kexdh_init();
-			break;
-
-		case SSH_MSG_NEWKEYS:
-			recv_msg_newkeys();
-			break;
-
+#endif
+			
 		default:
-			/* TODO this possibly should be handled */
-			TRACE(("unknown packet"));
+			TRACE(("unknown packet()"));
+			recv_unimplemented();
+			break;
 	}
-
-out:
-	buf_free(ses.payload);
-	ses.payload = NULL;
-
-	TRACE(("leave process_packet"));
 }
 
+/* This must be called directly after receiving the unimplemented packet.
+ * Isn't the most clean implementation, it relies on packet processing
+ * occurring directly after decryption. This is reasonably valid, since
+ * there is only a single decryption buffer */
+static void recv_unimplemented() {
+
+	CHECKCLEARTOWRITE();
+
+	buf_putbyte(ses.writepayload, SSH_MSG_UNIMPLEMENTED);
+	/* the decryption routine increments the sequence number, we must
+	 * decrement */
+	buf_putint(ses.writepayload, ses.recvseq - 1);
+
+	encrypt_packet();
+}
+	
+
+
 /* encrypt the writepayload, putting into writebuf, ready for write_packet()
  * to put on the wire */
 void encrypt_packet() {
@@ -648,11 +704,13 @@ static void buf_compress(buffer * dest, buffer * src, unsigned int len) {
 		}
 
 		assert(ses.keys->trans_zstream->avail_out == 0);
-		buf_resize(dest, ZLIB_COMPRESS_INCR);
+
+		/* the buffer has been filled, we must extend. This only happens in
+		 * unusual circumstances where the data grows in size after deflate(),
+		 * but it is possible */
+		buf_resize(dest, dest->size + ZLIB_COMPRESS_INCR);
 
 	}
 	TRACE(("leave buf_compress"));
 }
 #endif
-
-
