@@ -43,23 +43,29 @@
 #include "kex.h"
 #include "channel.h"
 
+/* need to know if the session struct has been initialised, this way isn't the
+ * cleanest, but works OK */
+int sessinitdone = 0;
+
+/* this is set when we get SIGINT or SIGTERM, the handler is in main.c */
+int exitflag = 0;
 
 static void session_init(int sock, runopts *opts, int childpipe,
-		struct sockaddr *remote_addr);
+		struct sockaddr *remoteaddr);
 static void session_identification();
 static void checktimeouts();
 
 struct sshsession ses;
 
 void child_session(int sock, runopts *opts, int childpipe,
-		struct sockaddr *remote_addr) {
+		struct sockaddr *remoteaddr) {
 
 	fd_set readfd, writefd;
 	struct timeval timeout;
 	int val;
 	
 	crypto_init();
-	session_init(sock, opts, childpipe, remote_addr);
+	session_init(sock, opts, childpipe, remoteaddr);
 
 	/* exchange identification, version etc */
 	session_identification();
@@ -92,6 +98,11 @@ void child_session(int sock, runopts *opts, int childpipe,
 		}
 		val = select(ses.maxfd+1, &readfd, &writefd, NULL, &timeout);
 		TRACE(("select val = %d", val));
+
+		if (exitflag) {
+			dropbear_exit("Terminated by signal");
+		}
+		
 		if (val < 0) {
 			if (errno == EINTR) {
 				continue;
@@ -159,37 +170,38 @@ void checktimeouts() {
 }
 
 
-#ifdef DOCLEANUP
-/* clean up session vars etc - only usually called at exit(), but useful
- * for checking memory left lying about */
+/* clean up a session on exit */
 void session_cleanup() {
 	
 	TRACE(("enter session_cleanup"));
-	chancleanup();
-
+	
+	/* we can't cleanup if we don't know the session state */
+	if (!sessinitdone) {
+		return;
+	}
+	
 	m_free(ses.session_id);
-	sign_key_free(ses.hostkey);
-	m_free(ses.remoteident);
-
-	buf_free(ses.readbuf);
-	buf_free(ses.decryptreadbuf);
-	buf_free(ses.payload);
-	buf_free(ses.writepayload);
+	if (ses.opts->hostkey) {
+		sign_key_free(ses.opts->hostkey);
+	}
 	m_free(ses.keys);
 
+	chancleanup();
 
 	TRACE(("leave session_cleanup"));
 }
-#endif
 
 /* called only at the start of a session, set up initial state */
 static void session_init(int sock, runopts *opts, int childpipe,
-		struct sockaddr *remote_addr) {
+		struct sockaddr *remoteaddr) {
 
 	struct timeval tv;
 	TRACE(("enter session_init"));
 
-	ses.remote_addr = remote_addr;
+	ses.remoteaddr = remoteaddr;
+
+	ses.addrstring = getaddrstring(remoteaddr);
+	ses.hostname = getaddrhostname(remoteaddr);
 
 	ses.sock = sock;
 	ses.maxfd = sock;
@@ -244,6 +256,8 @@ static void session_init(int sock, runopts *opts, int childpipe,
 	ses.transkexinit = NULL;
 	ses.dh_K = NULL;
 
+	sessinitdone = 1;
+
 	TRACE(("leave session_init"));
 }
 
