@@ -40,12 +40,11 @@
 #include "runopts.h"
 
 extern int sessinitdone;
+extern int exitflag;
 
-void child_session(int sock, runopts *opts, int childpipe,
-		struct sockaddr *remote_addr);
-#ifdef DOCLEANUP
 void session_cleanup();
-#endif
+void child_session(int sock, runopts *opts, int childpipe,
+		struct sockaddr *remoteaddr);
 
 struct key_context {
 
@@ -78,7 +77,9 @@ struct sshsession {
 	int childpipe; /* kept open until we successfully authenticate */
 	long connecttime; /* time of initial connection */
 
-	struct sockaddr *remote_addr;
+	struct sockaddr *remoteaddr; /* the host and port of the client */
+	unsigned char *addrstring; /* the text version of remoteaddr */
+	unsigned char *hostname; /* the remote hostname */
 
 	unsigned int maxfd; /* the maximum file descriptor to check with select() */
 	unsigned char *remoteident;
